/* Schleifenbauer data base tables and triggers, DM v1.24

        Updates v1.48 to v1.50
        1.50 fix issue with insertion of sensor data when cfnres = 1
*/


/* adjust data base name ! */
USE `sppdu`;


INSERT INTO dbhistory(version) VALUES("1.50");

DELIMITER $$$

DROP TRIGGER IF EXISTS `sp_process_raw`$$$
CREATE TRIGGER `sp_process_raw`
BEFORE INSERT on `spregister`
FOR EACH ROW
BEGIN
DECLARE reg INT;
DECLARE cfnrph INT;
DECLARE cfnrno INT;
DECLARE cfnrso INT;
DECLARE cfnrmo INT;
DECLARE cfamps INT;
DECLARE cfnrte INT;
DECLARE cfnres INT;
SET reg = new.register;
SET new.inserttime=CURRENT_TIMESTAMP;


/* the first parameter is the minimal delay between two inserts.
0 will prevent insertion of records and instead update records,

use a zero setting for static data (configuration) and non zero for dynamic data (measurements)
*/

/*
	first check config records to see how many channels are configured for this register
	limit the number of inserted record to number of channels or even zero

	1.40 add extended attribute
	1.50 fix issue with insertion of sensor data when cfnres = 1
*/

CASE reg
WHEN 100  THEN
BEGIN
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 0, 2, 1, 'idspdm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 2, 2, 1, 'idfwvs');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 4, 16, 1, 'idonbr');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 20, 16, 1, 'idpart');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 36, 16, 1, 'idsnbr');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 52, 2, 3, 'idchip');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 58, 2, 1, 'idaddr');
END;
WHEN 200  THEN
BEGIN
-- note: cfnr** must be static (interval 0)!
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 0, 1, 1, 'cfnrph');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 1, 1, 1, 'cfnrno');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 2, 1, 1, 'cfnrso');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 3, 1, 1, 'cfnrmo');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 4, 1, 1, 'cfamps');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 5, 1, 1, 'cfnrte');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 6, 1, 1, 'cfnres');
END; 
WHEN 300  THEN
BEGIN
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 0, 1, 1, 'ssstat');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 1, 1, 1, 'ssttri');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 2, 1, 1, 'ssitri');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 3, 1, 1, 'ssotri');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 4, 1, 1, 'ssvtri');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 5, 1, 1, 'ssftri');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 6, 1, 1, 'ssicda');
END;
WHEN 400  THEN 
BEGIN
END; 
WHEN 1000  THEN 
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrph' INTO cfnrph;
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrmo' INTO cfnrmo;
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrno' INTO cfnrno;
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrso' INTO cfnrso;
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 0, 16, 1, 'stdvnm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 16, 16, 1, 'stdvlc');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 32, 20, 1, 'stuser');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 52, 2, 1, 'stpkdr');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 54, 1, 1, 'strsal');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 56, 2, 1, 'stfodl');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 58, 1, 1, 'stpsav');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 59, 1, 1, 'stopom');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 60, 1, 1, 'stmaxt');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 61, 1, 1, 'stdiso');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'fd', 62, 2, cfnrph, 'stimcm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'fd', 68, 2, cfnrmo, 'stomcm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 122, 1, cfnrno, 'stomct');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 149, 1, cfnrph, 'stimct');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii',152, 8, cfnrph, 'stinnm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'ascii', 176, 8, cfnrno, 'stolnm');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 392, 2, cfnrso, 'stiodl');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 446, 1, 1, 'stcddt');
END;
WHEN 2000  THEN
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrso' INTO cfnrso;
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 0, 1, cfnrso, 'swocst');
	CALL  sp_insert_val(0, new.inserttime, new.address, new.data, new.extended, 'int', 27, 1, cfnrso, 'swosch');
END;
WHEN 3000  THEN
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrph' INTO cfnrph;
	CALL  sp_insert_val(60 * 60, new.inserttime, new.address, new.data, new.extended, 'int', 0, 3, cfnrph, 'imkwht');
	CALL  sp_insert_val(60 * 60, new.inserttime, new.address, new.data, new.extended, 'int', 9, 3, cfnrph, 'imkwhs');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 18, 2, cfnrph, 'impfac');
	CALL  sp_insert_val(60,          new.inserttime, new.address, new.data, new.extended, 'fd', 24, 2, cfnrph, 'imcrac');
	CALL  sp_insert_val(60,          new.inserttime, new.address, new.data, new.extended, 'fd', 30, 2, cfnrph, 'imcrpk');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 36, 2, cfnrph, 'imvoac');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 42, 2, cfnrph, 'imvodp');
END;
WHEN 4000  THEN
BEGIN
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrmo' INTO cfnrmo;
 	CALL  sp_insert_val(60 * 60, new.inserttime, new.address, new.data, new.extended, 'int',  0, 3, cfnrmo, 'omkwht');
	CALL  sp_insert_val(60 * 60, new.inserttime, new.address, new.data, new.extended, 'int', 81, 3, cfnrmo, 'omkwhs');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 162, 2, cfnrmo, 'ompfac');
 	CALL  sp_insert_val(60,          new.inserttime, new.address, new.data, new.extended, 'fd', 216, 2, cfnrmo, 'omcrac');
	CALL  sp_insert_val(60,          new.inserttime, new.address, new.data, new.extended, 'fd', 270, 2, cfnrmo, 'omcrpk');
	CALL  sp_insert_val(60 * 60,     new.inserttime, new.address, new.data, new.extended, 'fd', 324, 2, cfnrmo, 'omvoac');
END;
WHEN 5000  THEN
BEGIN
 DECLARE nrdigt int;
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnrte' INTO cfnrte;
 SELECT value FROM `splog` WHERE `pdu` = new.address AND `register` = 'cfnres' INTO cfnres;
 SELECT count(*) FROM `spactual` WHERE `pdu` = new.address AND `register` = 'snstyp' AND text = "T" INTO nrdigt;
	IF (cfnrte>0) THEN
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd', 0, 2, 1, 'pditem');
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd', 4, 2, 1, 'pdinpk');
	END IF;
	IF (cfnrte>1)  OR (cfnrte = 1 AND nrdigt > 0) THEN
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd', 2, 2, 1, 'pdetem');
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd', 6, 2, 1, 'pdexpk');
	END IF;
	IF (cfnres>0) THEN
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'ascii', 8, 1, cfnres, 'snstyp');
		CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'fd',   24, 2, cfnres, 'snsval');
        CALL  sp_insert_val(60, new.inserttime, new.address, new.data, new.extended, 'ascii', 56, 6, cfnres, 'snsnme');
	END IF;
END;
 ELSE
  BEGIN
  END;

END CASE;

SET new.done = 1;

END $$$
